# -*- Mode: python -*-
#
# audiofile.py - audio file handling
#
# Copyright (C) 2003, Xiph.org Foundation
# Copyright (C) 2003 Brett Smith <bretts@canonical.org>
#
# This file is part of positron.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of a BSD-style license (see the COPYING file in the
# distribution).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the license for more details.

from types import *
import os
from os import path
import string

import MP3Info
import ID3

def detect(filename):
    metadata = None

    for detect_func in detect_functions:
        metadata = detect_func(filename)
        if metadata != None:
            break
    else:
        metadata = None

    if metadata != None and metadata["title"] == None:
        # Must have title
        metadata["title"] = path.split(filename)[1]

    return metadata

def get_id3(filename, info = None):
    if not filename.lower().endswith('.mp3'):
        return None

    mp3info = ID3.ID3(filename)

    if info is None:
        info = { "type" : "mp3",
                 "size" : os.stat(filename)[6],
                 "length" : 0,
                 "tracknum" : None}

    info["title"] = mp3info.title
    info["artist"] = mp3info.artist
    info["album"] = mp3info.album
    try:
        info["tracknum"] = mp3info["TRACKNUMBER"]
    except KeyError:
        pass
    try:
        info["genre"] = mp3info.genres[mp3info.genre]
    except IndexError:
        info["genre"] = None

    return info

def detect_mp3(filename):
    try:
        info = get_id3(filename)
    except ID3.InvalidTagError:
        return None
    if info is None:
        return None
    try:
        f = file(filename, "rb+")
        mp3info = MP3Info.MP3Info(f)
        f.close()
        info["length"] = mp3info.mpeg.length
    except (MP3Info.Error, IOError, OSError):
        pass

    # Convert empty string entries to nulls
    for key in info.keys():
        if info[key] == "":
            info[key] = None

    return info

def detect_oggvorbis(filename):
    try:
        vf = ogg.vorbis.VorbisFile(filename)
        vc = vf.comment()
        vi = vf.info()        

        info = { "type" : "oggvorbis",
                 "size" : os.stat(filename).st_size,
                 "length" : int(vf.time_total(-1)),
                 "title" : None,
                 "artist" : None,
                 "album" : None,
                 "genre" : None,
                 "tracknumber" : None }

        actual_keys = map(string.lower, vc.keys())
        
        for tag in ("title","artist","album","genre","tracknumber"):
            if tag in actual_keys:
                value = vc[tag]
                # Force these to be single valued
                if type(value) == ListType or type(value) == TupleType:
                    value = value[0]

                # Convert from Unicode to ASCII since the Neuros can't
                # do Unicode anyway.
                #
                # I will probably burn in i18n hell for this.
                info[tag] = value.encode('ascii','replace')

    except ogg.vorbis.VorbisError:
        return None
    except IOError:
        return None
    except OSError:
        return None

    info['tracknum'] = info.get('tracknumber', None)

    return info


# Only put the ogg vorbis detection code in the list if
# we have the python module needed.

detect_functions = [detect_mp3]

try:
     import ogg.vorbis
     detect_functions.insert(0, detect_oggvorbis)
except ImportError:
    pass
